@@ -1,8 +1,8 @@
 #!/usr/bin/perl
 #
-# $Id: whereami.pl,v 1.29 2005/02/12 23:28:24 andrew Exp $
+# $Id: whereami.pl,v 1.33 2007/02/25 18:53:11 andrew Exp $
 #
-# by Andrew McMillan, Catalyst IT Ltd, (c) 2001,2002 licensed
+# by Andrew McMillan, Catalyst IT Ltd, (c) 2001-2005 licensed
 # for use under the Gnu GPL version 2
 #
 
@@ -75,7 +75,7 @@ print "BaseDir: $basedir\n" if ( $debug );
 $ENV{'BASEDIR'} =$basedir;
 $ENV{'STATEDIR'} =$statedir;
 $ENV{'LOCKDIR'} =$lockdir;
-$ENV{'PATH'} = "$scriptbase/tests:$scriptbase:/bin:/usr/bin:/sbin:/usr/sbin:$basedir/tests;/usr/local/share/whereami";
+$ENV{'PATH'} = "$scriptbase/tests:$scriptbase/actions:$scriptbase:/bin:/usr/bin:/sbin:/usr/sbin:$basedir/tests:/usr/local/share/whereami";
 $ENV{'DEBUGWHEREAMI'} = "1" if ( $debug || $scriptdebugging );
 
 
@@ -97,7 +97,7 @@ print "Last: $last_locations\n" if ( $debug );
 if ( "" eq "$now_locations" ) {
   if ( "" ne "$hint_locations" ) {
     # Add any hint locations into the starting mix
-    foreach( split( /[\s,]+/, $hint_locations ) ) {
+    foreach( split( /[,]+/, $hint_locations ) ) {
       printf ( "*hint**location: %s\n", $_) if ( $debug );
       $whereiam{$_} = $locn_seq++;
     }
@@ -109,7 +109,7 @@ if ( "" eq "$now_locations" ) {
 }
 else {
   # Then we just are wherever we were told on the command line.
-  foreach( split( /[\s,]+/, $now_locations ) ) {
+  foreach( split( /[,]+/, $now_locations ) ) {
     printf ( "*******location: %s\n", $_) if ( $debug );
     $whereiam{$_} = $locn_seq++;
   }
@@ -126,15 +126,16 @@ if ( $doactions ) {
   process_location("$basedir/whereami.conf", "$statedir/whereiam.sh");
 
   if ( $debug ) {
-    # Just show it to us
+    # Show it to us
     open( SCRIPTFILE, "< $statedir/whereiam.sh" ) or die_cleanly( "Can't read script file");
     while( <SCRIPTFILE> ) { print; }
     close SCRIPTFILE;
   }
-  else {
-    # Run the script we have constructed
-    system( "sh -c \". $statedir/whereiam.sh\"" );
-  }
+
+  # Run the script we have constructed
+  chmod 0755, "$statedir/whereiam.sh";
+  system( "sh", "-c", "$statedir/whereiam.sh" );
+
 }
 elsif ( $mapping) {
   # Print a list of mappings for ifupdown's mapping mechanism
@@ -314,6 +315,17 @@ sub save_locations {
   };
 }
 
+#############################################################
+## remove quotes from the parameter variable
+#############################################################
+sub remove_quotes {
+  my $in = shift;
+
+  $in =~ /^(['"])(.*)['"]$/ && do {
+    $in =~ /^[$1](.*)[$1]$/ && do { return $1; };
+  };
+  return $in;
+}
 
 ############################################################
 # Try and figure out where we actually are
@@ -347,9 +359,9 @@ my ( $confname ) = @_;
     # Trim leading blanks
     s/^\s+//;
 
-    /^echo\s+(.+)$/i && do {
+    /^echo\s+(\S.*)$/i && do {
       # Inform the nice user :-)
-      print "$1\n";
+      print remove_quotes($1), "\n";
       next;
     };
 
@@ -381,12 +393,12 @@ my ( $confname ) = @_;
     };
 
     # Deal to an 'if location[,...]' line
-    /^if (.*)$/i && do {
+    /^if\s+(\S.*)$/i && do {
       if ( $state =~ /^!?(if)|(else)/ ) {
         die_cleanly "$confname ($lno): 'if' may not be nested at this time";
       }
       $state = "!if";
-      foreach( split( /[\s,]/ , $1) ) {
+      foreach( split( /[,]+/ , $1) ) {
         if ( exists( $whereiam{$_} ) ) {
           $state = "if";
           last;
@@ -396,14 +408,14 @@ my ( $confname ) = @_;
     };
 
     # Deal to an 'elif location[,...]' line
-    /^elif (.*)$/i && do {
+    /^elif\s+(\S.*)$/i && do {
       if ( $state =~ /^(if)|(success)|(!else)/ ) {
         # If any 'if' has been successful then we skip everything
         $state = "!else";
         next;
       }
       $state = "!if";
-      foreach( split( /[\s,]/ , $1) ) {
+      foreach( split( /[,]/ , $1) ) {
         if ( exists( $whereiam{$_} ) ) {
           $state = "if";
           last;
@@ -426,9 +438,21 @@ my ( $confname ) = @_;
     syslog ( "debug",  "%#4d %-10.10s: %s\n", $lno, $state, $_ ) if ( $syslogging );
 
     # Strip the always, if it is there.
-    s/^always\s//i;
+    s/^always\s+//i;
 
-    ($testname, $params, $locations) = split(/\s+/, $_, 3);
+    # Now using a more complex process to split the line, so that we
+    # can handle spaces, quoting and commas inside the parameters.
+    s/^(\S+)\s*// && do { $testname = $1; };
+    s/\s*([[:alnum:].,_-]+)$// && do { $locations = $1; };
+    $params = $_;
+
+    printf( "testname[%s] parameters[%s], locations[%s]\n",
+                $testname, $params, $locations ) if($debug);
+
+    # get rid of quote characters in params.
+    # this is a bit tricky...
+    $params = remove_quotes($params);
+    printf("unquoted parameters: [%s]\n",$params) if($debug);
 
     # In future we might implement some of the commoner tests directly
     # in here.  Especially DHCP, where we can do one pass to resolve
@@ -443,39 +467,37 @@ my ( $confname ) = @_;
     }
     elsif ( $testname =~ /^default$/i ) {
       # The "default name" syntax doesn't change the processing state
-      $default = $params;
+      $default = $locations;
     }
     elsif ( $testname =~ /^at$/i ) {
       # Force a location
-      $locations = $params;
       $state = "success";
       printf ( "****************> Internal 'at' - adding locations: %s\n", $locations) if ( $debug );
       syslog ( "notice", "****************> Internal 'at' - adding locations: %s\n", $locations) if ( $syslogging );
 
-      foreach( split( /[\s,]+/, $locations ) ) {
+      foreach( split( /[,]+/, $locations ) ) {
         printf ( "*******location: %s\n", $_) if ( $debug );
         $whereiam{$_} = $locn_seq++;
       }
     }
     elsif ( $testname =~ /^notat$/i ) {
       # Force removal of a location
-      $locations = $params;
       $state = "success";
       printf ( "****************> Internal 'notat' - removing locations: %s\n", $locations) if ( $debug );
       syslog ( "notice", "****************> Internal 'notat' - removing locations: %s\n", $locations) if ( $syslogging );
 
-      foreach( split( /[\s,]+/, $locations ) ) {
+      foreach( split( /[,]+/, $locations ) ) {
         next if ( ! defined $whereiam{$_} );
         printf ( "******* not at : %s\n", $_) if ( $debug );
         delete $whereiam{$_};
       }
     }
-    elsif ( 0 eq system( "$testname $params" ) ) {
+    elsif ( 0 == system( $testname, $params ) ) {
       $state = "success";
       printf ( "****************> Test successful - adding locations: %s\n", $locations) if ( $debug );
       syslog ( "notice", "****************> Test successful - adding locations: %s\n", $locations) if ( $syslogging );
 
-      foreach( split( /[\s,]+/, $locations ) ) {
+      foreach( split( /[,]+/, $locations ) ) {
         printf ( "*******location: %s\n", $_) if ( $debug );
         $whereiam{$_} = $locn_seq++;
       }
@@ -525,37 +547,59 @@ my ( $confname, $scriptname ) = @_;
     s/^\s+//;
     ($condition, $script_line) = split( /\s+/, $_, 2);
     if ( $start_with eq "=" ) {
-      $found = 0;
-      while( ($k, $v) = each( %whereiam ) ) {
-        next if ( $found );
-        $condition  =~ /=$k$/ && do {
-          print "=>$condition<=|=>$k<|>$script_line" if ( $debug );
+      if ( $condition =~ /^=/ ) {
+        $found = 0;
+        while( ($k, $v) = each( %whereiam ) ) {
+          next if ( $found );
+          $condition  =~ /=$k$/ && do {
+            print "=>$condition<=|=>$k<|>$script_line" if ( $debug );
+            print WHEREIAM "$script_line";
+            syslog( "debug", "$condition $script_line") if ( $syslogging );
+            $found = 1;
+          };
+        }
+      }
+      elsif ( $condition =~ /^!/ ) {
+        my $not_locn = substr( $condition, 1);
+        if ( ! exists( $whereiam{$not_locn} ) ) {
           print WHEREIAM "$script_line";
+          print "=>$condition<=|=>$k<|>$script_line" if ( $debug );
           syslog( "debug", "$condition $script_line") if ( $syslogging );
-          $found = 1;
-        };
+        }
       }
     }
     else {
-      $found = 0;
-      while( ($k, $v) = each( %whereiwas ) ) {
-        next if ( $found );
-        $condition =~ /-$k$/ && do {
-          print "->$condition<-|->$k<|>$script_line" if ( $debug );
-          print WHEREIAM "$script_line";
-          syslog( "debug", "$condition $script_line") if ( $syslogging );
-          $found = 1;
-        } unless ( exists $whereiam{$k} );
+      if ( $condition =~ /^-/ ) {
+        $found = 0;
+        while( ($k, $v) = each( %whereiwas ) ) {
+          next if ( $found );
+          $condition =~ /-$k$/ && do {
+            print "->$condition<-|->$k<|>$script_line" if ( $debug );
+            print WHEREIAM "$script_line";
+            syslog( "debug", "$condition $script_line") if ( $syslogging );
+            $found = 1;
+          } unless ( exists $whereiam{$k} );
+        }
       }
-      $found = 0;
-      while( ($k, $v) = each( %whereiam ) ) {
-        next if ( $found );
-        $condition =~ /[+=]$k$/ && do {
-          print "+>$condition<+|+>$k<|>$script_line" if ( $debug );
+      elsif ( $condition =~ /^[+=]/ ) {
+        $found = 0;
+        while( ($k, $v) = each( %whereiam ) ) {
+          next if ( $found );
+          $condition =~ /[+=]$k$/ && do {
+            print "+>$condition<+|+>$k<|>$script_line" if ( $debug );
+            print WHEREIAM "$script_line";
+            syslog( "debug", "$condition $script_line") if ( $syslogging );
+            $found = 1;
+          } unless ( exists $whereiwas{$k} && $condition =~ /^[+]/ );
+        }
+      }
+      elsif ( $condition =~ /^!/ ) {
+        my $not_locn = substr( $condition, 1);
+        if ( ! exists( $whereiam{$not_locn} ) ) {
           print WHEREIAM "$script_line";
+          print "=>$condition<=|=>$k<|>$script_line" if ( $debug );
           syslog( "debug", "$condition $script_line") if ( $syslogging );
-          $found = 1;
-        } unless ( exists $whereiwas{$k} && $condition =~ /^[+]/ );
+        }
       }
     }
   }
